"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var fs_1 = __importDefault(require("fs"));
var typescript_1 = __importDefault(require("typescript"));
var options_1 = require("./options");
var LanguageServiceHost_1 = __importDefault(require("./LanguageServiceHost"));
var log_1 = require("./log");
var Engine = /** @class */ (function () {
    function Engine(cwd) {
        var configPath = path_1.default.resolve(cwd, 'tsconfig.json');
        var _a = typescript_1.default.readConfigFile(configPath, function (url) { return fs_1.default.readFileSync(url, { encoding: 'utf-8' }); }), config = _a.config, error = _a.error;
        if (error) {
            log_1.logError("rpt: Couldn't read tsconfig.json " + error.messageText);
            throw new Error('rpt: error');
        }
        var _b = typescript_1.default.convertCompilerOptionsFromJson(config.compilerOptions, cwd), options = _b.options, errors = _b.errors;
        if (errors.length !== 0) {
            for (var i = 0; i < errors.length; i += 1) {
                log_1.logError("rpt: Couldn't parse tsconfig " + errors[i].messageText);
            }
            throw new Error('rpt: error');
        }
        var compilerOptions = options_1.formatCompilerOptions(options);
        this._serviceHost = new LanguageServiceHost_1.default(compilerOptions, cwd);
        this._service = typescript_1.default.createLanguageService(this._serviceHost, typescript_1.default.createDocumentRegistry());
    }
    Engine.prototype.getCompilerOptions = function () {
        return this._serviceHost.getCompilationSettings();
    };
    Engine.prototype.transform = function (fileName, code) {
        var output = typescript_1.default.transpileModule(code, {
            fileName: fileName,
            compilerOptions: this._serviceHost.getCompilationSettings(),
            reportDiagnostics: true,
        });
        var diagnostics = output.diagnostics || [];
        var isHasError = log_1.printDiagnosticError(diagnostics);
        if (isHasError) {
            throw new Error('rpt: transform error');
        }
        return {
            code: output.outputText,
            map: output.sourceMapText ? JSON.parse(output.sourceMapText) : null,
        };
    };
    Engine.prototype.transformWithCheck = function (fileName, code) {
        this._serviceHost.setScriptSnapshot(fileName, code);
        var output = this._service.getEmitOutput(fileName);
        var diagnostics = this._service.getCompilerOptionsDiagnostics()
            .concat(this._service.getSyntacticDiagnostics(fileName))
            .concat(this._service.getSemanticDiagnostics(fileName));
        var isHasError = log_1.printDiagnosticError(diagnostics);
        if (isHasError) {
            throw new Error('rpt: transform error');
        }
        var outputFiles = output.outputFiles;
        var resultCode, resultMap;
        var declare, declareMap;
        for (var i = 0; i < outputFiles.length; i += 1) {
            if (/\.d\.ts$/.test(outputFiles[i].name)) {
                declare = outputFiles[i].text;
            }
            else if (/\.d\.ts\.map$/.test(outputFiles[i].name)) {
                declareMap = outputFiles[i].text;
            }
            else if (/\.map$/.test(outputFiles[i].name)) {
                resultMap = outputFiles[i].text;
            }
            else {
                resultCode = outputFiles[i].text;
            }
        }
        return {
            code: resultCode,
            map: resultMap ? JSON.parse(resultMap) : null,
        };
    };
    return Engine;
}());
exports.default = Engine;
